import {Injectable, OnDestroy} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {environment} from '../../environments/environment';
import {delay, map, shareReplay, tap} from 'rxjs/operators';
import {combineLatest, Observable, Subject, Subscription} from 'rxjs';
import {filterNullish} from '../../utils/rxjs-operators';
import {SorItem} from './sor.service';
import {PatientContextService} from './patient-context.service';
import {LocalStorageService} from './local-storage.service';

export interface UserContext {
  commonName: string;
  organizationName: string;
  eligibleRoles: UserAuthorizationType[];
  patientIdentifier?: string;
  organizationSor?: SorItem;
}

export interface UserAuthorizationType {
  authorizationCode: string;
  educationCode: string;
  educationType: string;
}

export interface KeepAliveContext {
  configuredIdleTimeout: number;
}

@Injectable({
  providedIn: 'root'
})
export class UserContextService implements OnDestroy {

  private fetchUserContextLocalStorage: Subscription;
  private USER_CONTEXT_PATH = '/usercontext';
  private LOGOUT_PATH = '/saml/Logout';
  private currentUser: UserContext | undefined;

  constructor(private http: HttpClient,
              private patientContext: PatientContextService,
              private userContextHolder: UserContextHolder,
              private localStorageService: LocalStorageService,
  ) {
    // Subscribe to make the observable hot so that any previous context will be loaded
    this.fetchUserContextLocalStorage =  this.userContext$.pipe(
      tap(userContext => this.setCommonName(userContext.commonName)),
      map(() => localStorageService.getUserContext())
      ).subscribe(
       localStorageContext => {
        if (localStorageContext?.role) {
          this.selectRole(localStorageContext.role);
        }
        if (localStorageContext?.organizationSor) {
          this.selectOrganizationSor(localStorageContext.organizationSor);
        }
      }
    );
  }
  public keepAlive$: Observable<KeepAliveContext> =
    this.http.get<KeepAliveContext>(environment.base_path + '/api' + this.USER_CONTEXT_PATH + '/keepAlive');

  public userContext$: Observable<UserContext> = this.http.get<UserContext>(environment.base_path + '/api' + this.USER_CONTEXT_PATH).pipe(
    tap((userContext: UserContext) => {
      this.currentUser = userContext;
      if (userContext.organizationSor){
        this.selectOrganizationSor(userContext.organizationSor);
      }
      if (userContext.patientIdentifier) {
        const {cpr} = this.localStorageService.getPatientContext();
        if (userContext.patientIdentifier !== cpr) {
          this.patientContext.setCpr(userContext.patientIdentifier);
        }
      }

      this.patientContext.loadPatientContextFromLocalStorage();
    }),
    delay(environment.mock ? 3000 : 0),
    shareReplay()
  );

  public eligibleRoles$: Observable<UserAuthorizationType[]> =
    this.userContext$.pipe(
    tap(userContext => {
      if (userContext.eligibleRoles && userContext.eligibleRoles.length === 1) {
        this.selectRole(userContext.eligibleRoles[0]);
      }
    }),
    map(userContext => userContext.eligibleRoles),
    shareReplay()
  );

  private innerSelectedRole: Subject<UserAuthorizationType> = new Subject();
  public selectedRole$: Observable<UserAuthorizationType> = this.innerSelectedRole
    .pipe(
      tap(selectedRole => console.log('selected role:' + JSON.stringify(selectedRole))),
      tap(selectedRole => this.localStorageService.setUserContextRole(selectedRole)),
      shareReplay()
    );

  private innerSelectedOrganizationSor: Subject<SorItem> = new Subject();
  public selectedOrganizationSor$: Observable<SorItem> = this.innerSelectedOrganizationSor
    .pipe(
      tap(selectedSor => console.log('selected organizationSor:' + selectedSor)),
      tap(selectedSor => this.localStorageService.setUserContextOrganization(selectedSor)),
      shareReplay()
    );

  public contextIsInitialized$: Observable<boolean> = combineLatest([
    this.userContext$,
    this.selectedRole$,
    this.selectedOrganizationSor$
  ]).pipe(
  filterNullish(),
  map(() => true )
  );

  public logout(): void {
    console.log('Logout triggered');
    this.patientContext.setCpr();
    this.localStorageService.clear();
    window.location.assign(environment.base_path + this.LOGOUT_PATH);
  }

  public keepAlive(): void {
    this.keepAlive$.subscribe();
  }

  public setCommonName(userName: string): void {
    this.userContextHolder._setName(userName);
    this.localStorageService.setUserContextCommonName(userName);
  }

  public selectRole( role: UserAuthorizationType): void {
    console.log('UserContextService selectRole() : ' + JSON.stringify(role));
    this.userContextHolder._setCurrentRole(role);
    this.innerSelectedRole.next(role);
  }

  public selectOrganizationSor(organizationSor: SorItem): void {
    console.log('UserContextService selectOrganizationSor() : ' + JSON.stringify(organizationSor));
    this.userContextHolder._setCurrentOrganization(organizationSor);
    this.innerSelectedOrganizationSor.next(organizationSor);
  }

  ngOnDestroy(): void {
    this.fetchUserContextLocalStorage.unsubscribe();
  }
}

@Injectable({
  providedIn: 'root'
})
export class UserContextHolder {
  private selectedName?: string;
  private selectedOrganization?: SorItem;
  private selectedRole?: UserAuthorizationType;

  constructor() {}

  get commonName(): string | undefined {
    return this.selectedName;
  }

  get currentOrganization(): SorItem | undefined {
    return this.selectedOrganization;
  }

  get currentRole(): UserAuthorizationType | undefined {
    return this.selectedRole;
  }

  public _setName(commonName: string): void {
    this.selectedName = commonName;
  }

  public _setCurrentOrganization(org: SorItem | undefined): void {
    this.selectedOrganization = org;
  }

  public _setCurrentRole(role: UserAuthorizationType | undefined): void {
    this.selectedRole = role;
  }
}
